//---------------------------------------------------------------------------

#pragma hdrstop

#include "mathmatrix.h"

Matrix::Matrix()
{
  this->MatrixType=mMatrix;
  this->Cells=0;
  this->RowCount=0;
  this->ColumnCount=0;
  this->EPS=0.000001;

  this->DetWasChanged=false;
  this->Det=0;
  ActiveDetCountingMode=ByEquation;
}
Matrix::Matrix(int RowSize, int ColumnSize)
{
  this->CreateMatrix(RowSize, ColumnSize);
  this->MatrixType=mMatrix;
  this->Clear();
  this->EPS=0.000001;
  this->DetWasChanged=false;
  this->Det=0;
  ActiveDetCountingMode=ByEquation;
}
Matrix::Matrix(int RowSize)
{
  this->CreateMatrix(RowSize, RowSize);
  this->MatrixType=mMatrix;
  this->Clear();
  this->EPS=0.000001;
  this->DetWasChanged=false;
  this->Det=0;
  ActiveDetCountingMode=ByEquation;
}
Matrix::Matrix(const Matrix& matrix)
{
  this->EPS=matrix.EPS;
  this->MatrixType=mMatrix;
  this->ActiveDetCountingMode=matrix.ActiveDetCountingMode;
  this->CreateMatrix(matrix.RowCount, matrix.ColumnCount);
  for(int i=0;i<matrix.RowCount;i++)
    for(int j=0;j<matrix.ColumnCount;j++)
      this->Cells[i][j]=matrix.Cells[i][j];
  if(!matrix.DetWasChanged)
  {
    this->Det=matrix.GetDet();
    this->DetWasChanged=false;
  }
  else
  {
    this->DetWasChanged=true;
  }
}
Matrix::Matrix(const Matrix* matrix)
{
  this->EPS=matrix->EPS;
  this->MatrixType=mMatrix;
  this->CreateMatrix(matrix->RowCount, matrix->ColumnCount);
  for(int i=0;i<matrix->RowCount;i++)
    for(int j=0;j<matrix->ColumnCount;j++)
      this->Cells[i][j]=matrix->Get(i,j);
  this->ActiveDetCountingMode=matrix->ByEquation;
  if(!matrix->DetWasChanged)
  {
    this->Det=matrix->GetDet();
    this->DetWasChanged=false;
  }
  else
  {
    this->DetWasChanged=true;
  }
}
Matrix::~Matrix()
{
  this->DestroyMatrix();
}
bool Matrix::Equals(const Matrix& matrix)
{
  if(this->RowCount!=matrix.RowCount||this->ColumnCount!=matrix.ColumnCount)
  throw invalid_argument("Comparison isn't allowed for matrixes with different sizes.");
  for(int i=0;i<matrix.RowCount;i++)
    for(int j=0;j<matrix.ColumnCount;j++)
      if(this->Cells[i][j]!=matrix.Cells[i][j])
        return false;
  return true;
}
double Matrix::Get(int RowNumber, int CellNumber)
{
  if(!this->Has(RowNumber,CellNumber))
    throw range_error("This cell is out of matrix range.");
  return this->Cells[RowNumber][CellNumber];
}
double Matrix::Get(int RowNumber)
{
  return this->Get(RowNumber,0);
}
bool Matrix::Set(int RowNumber, int CellNumber, double Value)
{
  if(!this->Has(RowNumber,CellNumber))
    throw range_error("This cell is out of matrix range.");
  if(this->Cells[RowNumber][CellNumber]!=Value)
  {
    this->Cells[RowNumber][CellNumber]=Value;
    this->DetWasChanged=true;
  }
  return true;
}
bool Matrix::Set(int RowNumber, double Value)
{
  return Set(RowNumber, 0, Value);
}
bool Matrix::Has(int RowNumber, int CellNumber)
{
  if(RowNumber>=this->RowCount||CellNumber>=this->ColumnCount)
    return false;
  else
    return true;
}
int Matrix::CellsCount()
{
   return this->RowCount*this->ColumnCount;
}
bool Matrix::IsSquare()
{
  return (this->RowCount==this->ColumnCount);
}
double Matrix::GetDet()
{
  if(this->RowCount==0||this->ColumnCount==0)
    throw logic_error("Unable to calculate determinate for the non-existing matrix.");
  if(this->RowCount != this->ColumnCount)
    throw logic_error("Unable to calculate determinate for the non-square matrix.");;
  if (this->DetWasChanged)
  {
    if(this->IsSteps())
    {
      this->Det = this->Cells[0][0];
      for (int i = 1; i < this->RowCount; i++)
        this->Det *= this->Cells[i][i];
    }
    else
    {
      switch (this->RowCount)
      {
        case 1:
          this->Det = this->Cells[0][0];
        break;
        case 2:
          this->Det = this->Cells[0][0] * this->Cells[1][1] - this->Cells[0][1] * this->Cells[1][0];
        break;
        case 3:
          this->Det = (this->Cells[0][0] * this->Cells[1][1] * this->Cells[2][2] + this->Cells[0][1] * this->Cells[1][2] * this->Cells[2][0] + this->Cells[1][0] * this->Cells[0][2] * this->Cells[2][1]) - (this->Cells[0][0] * this->Cells[1][2] * this->Cells[2][1] + this->Cells[0][1] * this->Cells[1][0] * this->Cells[2][2] + this->Cells[0][2] * this->Cells[1][1] * this->Cells[2][0]);
        break;
        default:
          Matrix* MatrixTriangle = this->ToSteps();
          if (MatrixTriangle == NULL || !MatrixTriangle->IsSteps())
            this->Det = 0;
          else
          {
            this->Det = MatrixTriangle->Get(0,0);
            for (int i = 1; i < MatrixTriangle->RowCount; i++)
              this->Det *= MatrixTriangle->Get(i,i);
          }
        break;
    }
    }
    this->DetWasChanged = false;
  }
  return Det;
}
bool Matrix::IsSteps()
{
for (int i = 0; i < this->RowCount; i++)
  for (int j = 0; j < i; j++)
    if (abs(this->Cells[i][j]) != 0)
      return false;
return true;
}
bool Matrix::IsIterable()
{
if (!this->IsSquare())
  return false;
double Count=0;
for (int i = 0; i < this->RowCount; i++)
{
  for (int j = 0; j < this->ColumnCount; j++)
  {
    Count += this->Cells[i][j];
  }
  if (Count >= 1)
    return false;
  Count = 0;
}
return true;
}
bool Matrix::IsSymmetric()
{
  if (!this->IsSquare())
    return false;
    for(int i=0; i<this->RowCount; i++)
      for(int j=0;j<this->ColumnCount;j++)
        if(i!=j && this->Cells[i][j]!=this->Cells[i][j])
          return false;
  return true;
}
void Matrix::RandomInt(int MaxValue)
{
  randomize();
  for (int i = 0; i < this->RowCount; i++)
  {
    for (int j = 0; j < this->ColumnCount; j++)
    {
      if(rand()%2)
        this->Cells[i][j] = random(MaxValue);
      else
        this->Cells[i][j] = -random(MaxValue);
    }
  }
}
void Matrix::RandomInt()
{
  randomize();
  for (int i = 0; i < this->RowCount; i++)
  {
    for (int j = 0; j < this->ColumnCount; j++)
    {
      if(rand()%2)
        this->Cells[i][j] = rand();
      else
        this->Cells[i][j] = -rand();
    }
  }
}
void Matrix::RandomDouble()
{
  randomize();
  for (int i = 0; i < this->RowCount; i++)
  {
    for (int j = 0; j < this->ColumnCount; j++)
    {
      if(rand()%2)
        this->Cells[i][j] = ((double)rand()/(double)rand());
      else
        this->Cells[i][j] = -((double)rand()/(double)rand());
    }
  }
}
void Matrix::Clear()
{
  for(int i=0;i<this->RowCount;i++)
    for(int j=0;j<this->ColumnCount;j++)
      this->Cells[i][j]=0;
}
Matrix* Matrix::Transpose()
{
  if(!this->IsSquare())
    throw logic_error("This matrix isn't square.");
  Matrix* MatrixResult=new Matrix(this->RowCount);
  for (int i = 0; i < MatrixResult->RowCount; i++)
    for (int j = 0; j < MatrixResult->ColumnCount; j++)
      MatrixResult->Cells[i][j] = this->Cells[j][i];
  return MatrixResult;
}
Matrix* Matrix::GetE()
{
  if(!this->IsSquare())
    throw logic_error("This matrix isn't square.");
  Matrix* MatrixResult=new Matrix(this->RowCount);
  for (int i = 0; i < MatrixResult->RowCount; i++)
  {
    for (int j = 0; j < MatrixResult->ColumnCount; j++)
    {
      if(i==j)
        MatrixResult->Cells[i][j] = 1;
      else
        MatrixResult->Cells[i][j] = 0;
    }
  }
  return MatrixResult;
}
Matrix* Matrix::GetE(double Value)
{
  if(!this->IsSquare())
    throw logic_error("This matrix isn't square.");
  Matrix* MatrixResult=new Matrix(this->RowCount);
  for (int i = 0; i < MatrixResult->RowCount; i++)
  {
    for (int j = 0; j < MatrixResult->ColumnCount; j++)
    {
      if(i==j)
        MatrixResult->Cells[i][j] = Value;
      else
        MatrixResult->Cells[i][j] = 0;
    }
  }
  return MatrixResult;
}

void Matrix::ToE()
{
  if(!this->IsSquare())
    throw logic_error("This matrix isn't square.");
  for (int i = 0; i < this->RowCount; i++)
  {
    for (int j = 0; j < this->ColumnCount; j++)
    {
      if(i==j)
        this->Cells[i][j] = 1;
      else
        this->Cells[i][j] = 0;
    }
  }
}
void __fastcall Matrix::ToE(double Value)
{
  if(!this->IsSquare())
    throw logic_error("This matrix isn't square.");
  for (int i = 0; i < this->RowCount; i++)
  {
    for (int j = 0; j < this->ColumnCount; j++)
    {
      if(i==j)
        this->Cells[i][j] = Value;
      else
        this->Cells[i][j] = 0;
    }
  }
}
std::string Matrix::ToString()
{
  std::string StrResult="";
  char buffer[80];
  for (int i = 0; i < this->RowCount; i++)
  {
    if(i>0)
      StrResult.append("\r\n");
    for (int j = 0; j < this->ColumnCount; j++)
    {
      if(j < this->ColumnCount-1)
        sprintf(buffer,"%f;",this->Cells[i][j]);
      else
        sprintf(buffer,"%f",this->Cells[i][j]);
      StrResult.append(buffer);
    }
  }
  //StrResult.append("\r\n");
  return StrResult;
}
char* Matrix::ToChar()
{
  std::string StrResult=this->ToString();
  char* buff=new char[StrResult.length()+5];
  strcpy(buff,StrResult.c_str());
  return buff;
}
//Operators overloaded
Matrix& Matrix::operator =(const Matrix& matrix)
{
  if(this == &matrix)
    return *this;
  this->DestroyMatrix();
  this->CreateMatrix(matrix.RowCount,matrix.ColumnCount);

  for(int i=0;i<matrix.RowCount;i++)
    for(int j=0;j<matrix.ColumnCount;j++)
      this->Cells[i][j]=matrix.Cells[i][j];
  if(!matrix.DetWasChanged)
  {
    this->Det=matrix.GetDet();
    this->DetWasChanged=false;
  }
  else
  {
    this->DetWasChanged=true;
  }
  return *this;
}
bool Matrix::operator ==(const Matrix& matrix)
{
   return this->Equals(matrix);
}
bool Matrix::operator !=(const Matrix& matrix)
{
   return !this->Equals(matrix);
}
Matrix* Matrix::operator +(const Matrix& matrix)
{
if (this->RowCount != matrix.RowCount || this->ColumnCount != matrix.ColumnCount)
  throw invalid_argument("Summation isn't allowed for matrixes with different sizes.");
Matrix* MatrixResult=new Matrix(matrix.RowCount, matrix.ColumnCount);
for (int i = 0; i < matrix.RowCount; i++)
  for (int j = 0; j < matrix.ColumnCount; j++)
    MatrixResult->Set(i, j, this->Get(i,j) + matrix.Get(i,j));
return MatrixResult;
}
Matrix* Matrix::operator -(const Matrix& matrix)
{
if (this->RowCount != matrix.RowCount || this->ColumnCount != matrix.ColumnCount)
  throw invalid_argument("Substraction isn't allowed for matrixes with different sizes.");
Matrix* MatrixResult=new Matrix(matrix.RowCount, matrix.ColumnCount);
for (int i = 0; i < matrix.RowCount; i++)
  for (int j = 0; j < matrix.ColumnCount; j++)
    MatrixResult->Set(i,j, this->Get(i,j) - matrix.Get(i,j));
return MatrixResult;
}
Matrix* Matrix::operator *(const Matrix& matrix)
{
if (this->ColumnCount != matrix.RowCount)
  throw invalid_argument("Multiplication isn't allowed for this matrixes.");
Matrix* MatrixResult=new Matrix(this->RowCount, matrix.ColumnCount);
for (int i = 0; i < this->RowCount; i++)
{
  for (int j = 0; j < matrix.ColumnCount; j++)
  {
    MatrixResult->Set(i, j, 0);
    for (int k = 0; k < matrix.RowCount; k++)
      MatrixResult->Set(i, j, MatrixResult->Get(i, j)+this->Get(i, k) * matrix.Get(k, j));
  }
}
return MatrixResult;
}
Matrix* Matrix::operator *(const double value)
{
Matrix* MatrixResult=new Matrix(this->RowCount, this->ColumnCount);
for (int i = 0; i < MatrixResult->RowCount; i++)
  for (int j = 0; j < MatrixResult->ColumnCount; j++)
    MatrixResult->Set(i, j, this->Get(i,j)*value);
return MatrixResult;
}
Matrix* Matrix::operator /(const Matrix& matrix)
{
if (this->RowCount != matrix.RowCount || this->ColumnCount != matrix.ColumnCount)
  throw invalid_argument("Division isn't allowed for matrixes with different sizes.");
Matrix* MatrixResult=new Matrix(this->RowCount, this->ColumnCount);
MatrixResult=(*this)*(*matrix.Inverse());
return MatrixResult;
}
Matrix* Matrix::operator /(const double value)
{
if(value==0)
  throw logic_error("Unable to divide by zero.");
Matrix* MatrixResult=new Matrix(this->RowCount, this->ColumnCount);
for (int i = 0; i < MatrixResult->RowCount; i++)
  for (int j = 0; j < MatrixResult->ColumnCount; j++)
    MatrixResult->Set(i, j, this->Get(i,j)/value);
return MatrixResult;
}
//Create'n'Destroy
void Matrix::CreateMatrix(int RowSize, int ColumnSize)
{
  this->Cells=new double*[RowSize];
  for(int j=0; j<RowSize; j++)
    this->Cells[j] = new double[ColumnSize];
  this->RowCount=RowSize;
  this->ColumnCount=ColumnSize;
}
void Matrix::DestroyMatrix()
{
  if(this->Cells!=0)
  {
    for (int i = 0; i < this->RowCount; i++)
       delete[] this->Cells[i];
    delete[] this->Cells;
  }
}
Matrix* Matrix::ToSteps()
{
if(!this->IsSquare())
  throw logic_error("This matrix isn't square.");
double kk = 0;
Matrix* MatrixResult=new Matrix(this);
for (int i = 1; i < this->RowCount; i++)
{
  for (int j = 0; j < i; j++)
  {
    if(this->Cells[j][j]==0)
    {
      MatrixResult->DetWasChanged=false;
      break;
    }
    if(MatrixResult->Get(j,j)==0)
      continue;
    kk = MatrixResult->Get(i,j) / MatrixResult->Get(j,j);
    for (int k = j; k < MatrixResult->ColumnCount; k++)
    {
      MatrixResult->Set(i, k, MatrixResult->Get(i,k) - MatrixResult->Get(j,k) * kk);
      //if(abs(MatrixResult->Get(i,k))<MatrixResult->EPS)
      //  MatrixResult->Set(i,k,0);
    }
  }
}
return MatrixResult;
}
double Matrix::Fourier(double x)
{

if(this->RowCount!=2||this->ColumnCount <= 2)
  throw logic_error("Unable to use this matrix for Fourier approximation.");
double fCurr(FourierElemAn(0)/2),fPrev(0);
double l=abs(abs(this->Cells[0][0]) - abs(this->Cells[0][this->ColumnCount-1]));
int i=0;
do
{
  i++;
  fPrev=fCurr;
  fCurr+=FourierElemAn(i)*cos(M_PI*i*x/l) + FourierElemBn(i)*sin(M_PI*i*x/l);
} while(abs(abs(fCurr)-abs(fPrev)) < this->EPS);
return fCurr;
}
double Matrix::FourierElemAn(int n)
{
double ElemAnResult=0;

if(!n)
{
  for(int i=0; i < this->ColumnCount - 2; i++)
    ElemAnResult+=this->Cells[1][i]+this->Cells[1][i+1];
  ElemAnResult/=this->ColumnCount;
}
else
{
  for(int i=0; i < this->ColumnCount - 2; i++)
    ElemAnResult+=this->Cells[1][i]*cos((2*M_PI*n*i)/this->ColumnCount)+this->Cells[1][i+1]*cos((2*M_PI*n*(i+1))/this->ColumnCount);
  ElemAnResult/=this->ColumnCount;
}
return ElemAnResult;
}
double Matrix::FourierElemBn(int n)
{
double ElemAnResult=0;
for(int i=0; i < this->ColumnCount - 2; i++)
  ElemAnResult+=this->Cells[1][i]*sin((2*M_PI*n*i)/this->ColumnCount)+this->Cells[1][i+1]*sin((2*M_PI*n*(i+1))/this->ColumnCount);
ElemAnResult/=this->ColumnCount;
return ElemAnResult;
}
void Matrix::ExpantionLU(Matrix& MatrixL, Matrix& MatrixU)
{
if (!this->IsSquare())
  throw logic_error("This matrix isn't square.");
if(!MatrixL.IsSquare()||MatrixL.RowCount!=this->RowCount)
  throw logic_error("Matrix L has the different size with the source matrix.");
if(!MatrixU.IsSquare()||MatrixU.RowCount!=this->RowCount)
  throw logic_error("Matrix U has the different size with the source matrix.");
for (int i = 0; i < this->RowCount; i++)
{
  for (int j = 0; j < this->ColumnCount; j++)
  {
    //u[i,j]
    MatrixU.Set(i,j,this->Get(i, j));
    for (int k = 0; k <= i - 1; k++)
      MatrixU.Set(i, j,MatrixU.Get(i,j)-(MatrixL.Get(i,k) * MatrixU.Get(k, j)));
    //l[i,j]
    MatrixL.Set(i, j, this->Get(i, j));
    for (int k = 0; k <= j - 1; k++)
      MatrixL.Set(i, j, MatrixL.Get(i, j)-(MatrixL.Get(i, k) * MatrixU.Get(k, j)));
    if (MatrixU.Get(j, j) != 0)
      MatrixL.Set(i, j, MatrixL.Get(i, j)/MatrixU.Get(j, j));
  }
}
for (int i = 0; i < MatrixU.RowCount; i++)
{
  for (int j = 0; j < MatrixU.ColumnCount; j++)
  {
    if (i > j)
      MatrixU.Set(i, j, 0);
    else if (i == j)
      MatrixL.Set(i, j, 1);
    else if (i < j)
      MatrixL.Set(i, j, 0);
  }
}
}
Matrix* Matrix::ByGauss(const Matrix& MatrixA, const Matrix& MatrixB)
{
  Matrix* B1=new Matrix(MatrixB);
  Matrix* A1 = MatrixA.ToGaussSteps(B1);
  if (!A1->IsSquare())
    throw logic_error("This matrix isn't square.");
  Matrix* MatrixRes = new Matrix(B1);
  for (int i = MatrixRes->RowCount-1; i >= 0; i--)
  {
    for (int j = MatrixRes->RowCount - 1; j > i; j--)
    {
      MatrixRes->Set(i, 0, MatrixRes->Get(i, 0)-A1->Get(i, j)*MatrixRes->Get(j, 0));
    }
    MatrixRes->Set(i, 0, MatrixRes->Get(i, 0)/A1->Get(i, i));
  }
  return MatrixRes;
}
Matrix* Matrix::ByGauss(const Matrix& MatrixB)
{
  Matrix* B1=new Matrix(MatrixB);
  Matrix* A1 = this->ToGaussSteps(B1);
  if (!A1->IsSquare())
    throw logic_error("This matrix isn't square.");
  Matrix* MatrixRes = new Matrix(B1);
  for (int i = MatrixRes->RowCount-1; i >= 0; i--)
  {
    for (int j = MatrixRes->RowCount - 1; j > i; j--)
    {
      MatrixRes->Set(i, 0, MatrixRes->Get(i, 0)-A1->Get(i, j)*MatrixRes->Get(j, 0));
    }
    MatrixRes->Set(i, 0, MatrixRes->Get(i, 0)/A1->Get(i, i));
  }
  return MatrixRes;
}
Matrix* Matrix::ToGaussSteps(Matrix* MatrixB)
{
  double kk = 0;
  Matrix* MatrixRes=new Matrix(this);
  for (int i = 1; i < this->RowCount; i++)
  {
    for (int j = 0; j < i; j++)
    {
      if(MatrixRes->Get(j,j) == 0)
          throw logic_error("Unable to made steps from this matrix");
      kk = MatrixRes->Get(i,j) / MatrixRes->Get(j,j);
      for (int k = j; k < MatrixRes->ColumnCount; k++)
        MatrixRes->Set(i, k, MatrixRes->Get(i,k) - MatrixRes->Get(j,k) * kk);
      MatrixB->Set(i, 0, MatrixB->Get(i, 0)-MatrixB->Get(j, 0) * kk);
    }
  }
  return MatrixRes;
}
Matrix* Matrix::ByBackGauss(const Matrix& MatrixA, const Matrix& MatrixB)
{
  if (!MatrixA.IsSquare())
    throw logic_error("Matrix A isn't square.");
  if (!MatrixA.Transpose()->IsSteps())
    throw logic_error("Matrix A isn't step");
  Matrix* MatrixRes = new Matrix(MatrixB.RowCount, 1);
  for(int i=0; i<MatrixRes->RowCount; i++)
  {
    MatrixRes->Set(i,0, MatrixB.Get(i, 0));
    for (int j = 0; j < i; j++)
    {
      MatrixRes->Set(i, 0, MatrixRes->Get(i, 0)-MatrixA.Get(i, j)*MatrixRes->Get(j, 0));
    }
    MatrixRes->Set(i, 0,  MatrixRes->Get(i, 0)/MatrixA.Get(i, i));
  }
  return MatrixRes;
}
Matrix* Matrix::ByBackGauss(const Matrix& MatrixB)
{
  if (!this->IsSquare())
    throw logic_error("Matrix A isn't square.");
  if (!this->Transpose()->IsSteps())
    throw logic_error("Matrix A isn't step");
  Matrix* MatrixRes = new Matrix(MatrixB.RowCount, 1);
  for(int i=0; i<MatrixRes->RowCount; i++)
  {
    MatrixRes->Set(i,0, MatrixB.Get(i, 0));
    for (int j = 0; j < i; j++)
    {
      MatrixRes->Set(i, 0, MatrixRes->Get(i, 0)-this->Get(i, j)*MatrixRes->Get(j, 0));
    }
    MatrixRes->Set(i, 0,  MatrixRes->Get(i, 0)/this->Get(i, i));
  }
  return MatrixRes;
}
Matrix* Matrix::ByBackGaussL(const Matrix& MatrixA, const Matrix& MatrixB)
{
  if (!MatrixA.IsSquare())
    throw logic_error("Matrix A isn't square.");
  if (!MatrixA.Transpose()->IsSteps())
    throw logic_error("Matrix A isn't step");
  Matrix* MatrixRes = new Matrix(MatrixB.RowCount, 1);
  for (int i = 0; i < MatrixRes->RowCount; i++)
  {
    MatrixRes->Set(i, 0, MatrixB.Get(i, 0));
    for (int j = 0; j < i; j++)
    {
      MatrixRes->Set(i, 0, MatrixRes->Get(i, 0)-MatrixA.Get(i, j)*MatrixRes->Get(j, 0));
    }
  }
  return MatrixRes;
}
Matrix* Matrix::ByBackGaussL(const Matrix& MatrixB)
{
  if (!this->IsSquare())
    throw logic_error("Matrix A isn't square.");
  if (!this->Transpose()->IsSteps())
    throw logic_error("Matrix A isn't step");
  Matrix* MatrixRes = new Matrix(MatrixB.RowCount, 1);
  for (int i = 0; i < MatrixRes->RowCount; i++)
  {
    MatrixRes->Set(i, 0, MatrixB.Get(i, 0));
    for (int j = 0; j < i; j++)
    {
      MatrixRes->Set(i, 0, MatrixRes->Get(i, 0)-this->Get(i, j)*MatrixRes->Get(j, 0));
    }
  }
  return MatrixRes;
}
Matrix* Matrix::BySqrt(const Matrix& MatrixA, const Matrix& MatrixB)
{
  if(!MatrixA.IsSymmetric())
    throw logic_error("Matrix A isn't symmetric. you can't use SQRT method for this matrix");
  Matrix* L1 = new Matrix(MatrixA.RowCount);
  Matrix* U1 = new Matrix(MatrixA.RowCount);
  MatrixA.ExpantionLU(L1, U1);
  Matrix* MatrixRes = ByGauss(U1, ByBackGaussL(L1, MatrixB));
  return MatrixRes;
}
Matrix* Matrix::BySqrt(const Matrix& MatrixB)
{
  if(!this->IsSymmetric())
    throw logic_error("Matrix A isn't symmetric. you can't use SQRT method for this matrix");
  Matrix* L1 = new Matrix(this->RowCount);
  Matrix* U1 = new Matrix(this->RowCount);
  this->ExpantionLU(L1, U1);
  Matrix* MatrixRes = U1->ByGauss(ByBackGaussL(L1, MatrixB));
  return MatrixRes;
}
double Matrix::Cofactor(int RowNumber, int CellNumber)
{
if(!this->IsSquare())
  throw logic_error("This matrix isn't square.");
Matrix* MatrixResult=new Matrix(this->RowCount-1);
for (int i = 0; i < this->RowCount-1; i++)
{
  for (int j = 0; j < this->ColumnCount-1; j++)
  {
     if(i<RowNumber&&j<CellNumber)
     {
       MatrixResult->Set(i, j, this->Cells[i][j]);
     }
     else if(i<RowNumber&&j>=CellNumber)
     {
       MatrixResult->Set(i, j, this->Cells[i][j+1]);
     }
     else if(i>=RowNumber&&j<CellNumber)
     {
       MatrixResult->Set(i, j, this->Cells[i+1][j]);
     }
     else if(i>=RowNumber&&j>=CellNumber)
     {
       MatrixResult->Set(i, j, this->Cells[i+1][j+1]);
     }
  }
}
if ((RowNumber+CellNumber) % 2 == 0)
  return MatrixResult->GetDet();
else
  return -(MatrixResult->GetDet());
}
double Matrix::Minor(int RowNumber, int CellNumber)
{
if(!this->IsSquare())
  throw logic_error("This matrix isn't square.");
Matrix* MatrixResult=new Matrix(this->RowCount-1);
for (int i = 1; i < this->RowCount-1; i++)
{
  for (int j = 0; j < this->ColumnCount-1; j++)
  {
     if(i<RowNumber&&j<CellNumber)
       MatrixResult->Set(i, j, this->Cells[i][j]);
     else if(i<RowNumber&&j>CellNumber)
       MatrixResult->Set(i, j, this->Cells[i][j+1]);
     else if(i>RowNumber&&j<CellNumber)
       MatrixResult->Set(i, j, this->Cells[i+1][j]);
     else if(i>RowNumber&&j>CellNumber)
       MatrixResult->Set(i, j, this->Cells[i+1][j+1]);
  }
}
return MatrixResult->GetDet();
}
void Matrix::ClearByEPS()
{
for(int i=0;i<this->RowCount;i++)
{
  for(int j=0;j<this->ColumnCount;j++)
  {
     if(fabs(this->Cells[i][j])<this->EPS)
     {
       this->Cells[i][j]=0;
     }
  }
}
}
void Matrix::ClearByEPS(double EPSValue)
{
if(EPSValue>=0)
  this->EPS=EPSValue;
this->ClearByEPS();
}
Matrix* Matrix::Inverse()
{
if(!this->IsSquare())
  throw logic_error("This matrix isn't square.");
double CurrDet=this->GetDet();
if(CurrDet==0)
  throw logic_error("Matrix determinant is null. Matrix can't be inverted.");
Matrix* MatrixResult=new Matrix(this->RowCount);
switch(ActiveDetCountingMode)
{
  case ByCofactor:
  for(int i=0;i<this->RowCount;i++)
  {
    for(int j=0;j<this->ColumnCount;j++)
    {
      MatrixResult->Set(j, i, this->Cofactor(i, j)/CurrDet);
    }
  }
  break;
  case ByEquation:
  Matrix* CurrB = new Matrix(this->RowCount, 1);
  for(int j=0; j < this->RowCount; j++)
  {
    if(j>0)
      CurrB->Set(j-1, 0, 0);
    CurrB->Set(j, 0, 1);
    Matrix* CurrX = this->ByGauss(CurrB);
    for(int i=0; i < this->ColumnCount; i++)
      MatrixResult->Set(i, j, CurrX->Get(i, 0));
    delete CurrX;
    CurrX=NULL;
  }
  delete CurrB;
  break;
}
return MatrixResult;
}
Matrix* Matrix::WithControlColumn()
{
if(!this->IsSquare())
  throw logic_error("This matrix isn't square.");
Matrix* MatrixResult=new Matrix(this->RowCount,this->ColumnCount+1);
int CurrRowSum=0;
for(int i=0;i<this->RowCount;i++)
{
  for(int j=0;j<this->ColumnCount;j++)
  {
    MatrixResult->Set(i, j, this->Cells[i][j]);
    CurrRowSum+=this->Cells[i][j];
  }
  MatrixResult->Set(i, this->ColumnCount, CurrRowSum);
  CurrRowSum=0;
}
return MatrixResult;
}
double Matrix::ArithmeticMean()
{
  double Result=0;
  for(int i=0;i<this->RowCount;i++)
    for(int j=0;j<this->ColumnCount;j++)
      Result+=this->Cells[i][j];
  return Result/this->CellsCount();
}
//Matrix Single
MatrixSingle::MatrixSingle()
{
  this->MatrixType=mSingle;
}
MatrixSingle::MatrixSingle(double dValue)
{
  this->MatrixType=mSingle;
  this->Value=dValue;
}
void MatrixSingle::RandomDouble()
{
  randomize();
  this->Value=((double)rand()/(double)rand());
}
void MatrixSingle::RandomInt(int MaxValue)
{
  randomize();
  this->Value=random(MaxValue);
}
void MatrixSingle::RandomInt()
{
  randomize();
  this->Value=rand();
}
double MatrixSingle::GetDet()
{
  return this->Value;
}
double MatrixSingle::ToDouble()
{
  return this->Value;
}
//Operators overloaded
bool MatrixSingle::Equals(const MatrixSingle& matrix)
{
  return this->Value==matrix.Value;
}
MatrixSingle& MatrixSingle::operator =(const MatrixSingle& matrix)
{
  this->Value=matrix.Value;
  return *this;
}
bool MatrixSingle::operator ==(const MatrixSingle& matrix)
{
   return this->Equals(matrix);
}
bool MatrixSingle::operator !=(const MatrixSingle& matrix)
{
   return !this->Equals(matrix);
}
MatrixSingle* MatrixSingle::operator +(const MatrixSingle& matrix)
{
  return new MatrixSingle(this->Value+matrix.Value);
}
MatrixSingle* MatrixSingle::operator -(const MatrixSingle& matrix)
{
  return new MatrixSingle(this->Value-matrix.Value);
}
MatrixSingle* MatrixSingle::operator *(const MatrixSingle& matrix)
{
  return new MatrixSingle(this->Value*matrix.Value);
}
MatrixSingle* MatrixSingle::operator *(const double value)
{
  return new MatrixSingle(this->Value*value);
}
MatrixSingle* MatrixSingle::operator /(const MatrixSingle& matrix)
{
  return new MatrixSingle(this->Value/matrix.Value);
}
std::string MatrixSingle::ToString()
{
  std::string StrResult="";
  char* buffer=new char[80];
  sprintf(buffer,"%f\)",this->Value);
  StrResult.append(buffer);
  return StrResult;
}
char* MatrixSingle::ToChar()
{
  char* buffer=new char[80];
  sprintf(buffer,"%f\0",this->Value);
  return buffer;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//Additional
//---------------------------------------------------------------------------
double __fastcall MatrixMultiplication2(const Matrix* matrix1, const Matrix* matrix2)
{
double result=0;
if (matrix1->RowCount != matrix2->RowCount)
  throw invalid_argument("The matrixes have to be the same size.");
for(int i=0; i<matrix1->RowCount; i++)
  result+=matrix1->Get(i)*matrix2->Get(i);
return result;
}

#pragma package(smart_init)
