//---------------------------------------------------------------------------

#ifndef mathmatrixH
#define mathmatrixH

#include <stdlib.h>
#include <string.h>
#include <stdexcept.h>
#include <time.h>
#include <stdio.h>


#include <vector>
enum MatrixClass {
    mMatrix,
    mSingle
  };
class MatrixObject
{
  private:
  public:

  void RandomDouble(){};
  void RandomInt(int MaxValue){};
  void RandomInt(){};

  double GetDet(){return 0;};

  double EPS;

  std::string ToString(){return "\0";};
  char* ToChar(){return "\0";};

  enum MatrixClass MatrixType;
};
class MatrixSingle : public MatrixObject
{
  private:
  public:
    MatrixSingle();
    MatrixSingle(double dValue);

    bool Equals(const MatrixSingle& matrix);

    void RandomDouble();
    void RandomInt(int MaxValue);
    void RandomInt();

    double GetDet();

    MatrixSingle& operator =(const MatrixSingle& matrix);
    bool operator ==(const MatrixSingle& matrix);
    bool operator !=(const MatrixSingle& matrix);
    MatrixSingle* operator +(const MatrixSingle& matrix);
    MatrixSingle* operator -(const MatrixSingle& matrix);
    MatrixSingle* operator *(const MatrixSingle& matrix);
    MatrixSingle* operator *(const double value);
    MatrixSingle* operator /(const MatrixSingle& matrix);

    double ToDouble();
    double Value;

    std::string ToString();
    char* ToChar();
};
class Matrix : public MatrixObject
{
//new matrix params
  public:
    Matrix();
    Matrix(int RowSize, int ColumnSize);
    Matrix(int RowSize);
    Matrix(const Matrix& matrix);
    Matrix(const Matrix* matrix);
    ~Matrix();

    bool Equals(const Matrix& matrix);

    double Get(int RowNumber, int CellNumber);
    double Get(int RowNumber);
    bool Set(int RowNumber, int CellNumber,double Value);
    bool Set(int RowNumbe,double Value);
    bool Has(int RowNumber, int CellNumber);

    int RowCount;
    int ColumnCount;
    int CellsCount();
    bool IsSquare();
    bool IsSteps();
    bool IsIterable();
    bool IsSymmetric();

    Matrix* ToSteps();
    double Matrix::Fourier(double x);
    double Matrix::FourierElemAn(int n);
    double Matrix::FourierElemBn(int n);
    void ExpantionLU(Matrix& MatrixL, Matrix& MatrixU);
    Matrix* ByGauss(const Matrix& MatrixA, const Matrix& MatrixB);
    Matrix* ByGauss(const Matrix& MatrixB);
    Matrix* ToGaussSteps(Matrix* MatrixB);
    Matrix* ByBackGauss(const Matrix& MatrixA, const Matrix& MatrixB);
    Matrix* ByBackGauss(const Matrix& MatrixB);
    Matrix* ByBackGaussL(const Matrix& MatrixA, const Matrix& MatrixB);
    Matrix* ByBackGaussL(const Matrix& MatrixB);
    Matrix* BySqrt(const Matrix& MatrixA, const Matrix& MatrixB);
    Matrix* BySqrt(const Matrix& MatrixB);
    //  (, )
    double Cofactor(int RowNumber, int CellNumber);
    double Minor(int RowNumber, int CellNumber);
    double GetDet();
    Matrix* Inverse();
    Matrix* WithControlColumn();

    void RandomDouble();
    void RandomInt(int MaxValue);
    void RandomInt();
    void Clear();
    void ClearByEPS();
    void ClearByEPS(double EPSValue);
    Matrix* Transpose();
    Matrix* GetE();
    Matrix* GetE(double Value);
    void ToE();
    void __fastcall ToE(double Value);
    double ArithmeticMean();

    std::string ToString();
    char* ToChar();

    Matrix& operator =(const Matrix& matrix);
    bool operator ==(const Matrix& matrix);
    bool operator !=(const Matrix& matrix);
    Matrix* operator +(const Matrix& matrix);
    Matrix* operator -(const Matrix& matrix);
    Matrix* operator *(const Matrix& matrix);
    Matrix* operator *(const double value);
    Matrix* operator /(const Matrix& matrix);
    Matrix* operator /(const double value);

    enum DetCountingMode{
    ByCofactor,
    ByEquation
    } ActiveDetCountingMode;

  private:
    bool DetWasChanged;
    double Det;
    double** Cells;

    void CreateMatrix(int RowSize, int ColumnSize);
    void DestroyMatrix();
};

//---------------------------------------------------------------------------
//Additional
//---------------------------------------------------------------------------
double __fastcall MatrixMultiplication2(const Matrix* matrix1, const Matrix* matrix2);
//---------------------------------------------------------------------------
#endif
